"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Linker = void 0;
const ParticlesBase_1 = require("./ParticlesBase");
const Utils_1 = require("../../Utils");
class Linker extends ParticlesBase_1.ParticlesBase {
    constructor(container) {
        super(container, "linker");
    }
    isEnabled(particle) {
        return particle.options.links.enable;
    }
    reset() {
    }
    interact(p1) {
        var _a;
        const container = this.container;
        const linkOpt1 = p1.options.links;
        const optOpacity = linkOpt1.opacity;
        const optDistance = (_a = p1.linksDistance) !== null && _a !== void 0 ? _a : container.retina.linksDistance;
        const canvasSize = container.canvas.size;
        const warp = linkOpt1.warp;
        const pos1 = p1.getPosition();
        const range = warp
            ? new Utils_1.CircleWarp(pos1.x, pos1.y, optDistance, canvasSize)
            : new Utils_1.Circle(pos1.x, pos1.y, optDistance);
        const query = container.particles.quadTree.query(range);
        for (const p2 of query) {
            const linkOpt2 = p2.options.links;
            if (p1 === p2 || !linkOpt2.enable || linkOpt1.id !== linkOpt2.id || p2.spawning || p2.destroyed) {
                continue;
            }
            const pos2 = p2.getPosition();
            const distance = this.getDistance(pos1, pos2, optDistance, canvasSize, warp && linkOpt2.warp);
            if (distance > optDistance) {
                return;
            }
            const opacityLine = (1 - distance / optDistance) * optOpacity;
            this.setColor(p1);
            if (p2.links.map((t) => t.destination).indexOf(p1) === -1 &&
                p1.links.map((t) => t.destination).indexOf(p2) === -1) {
                p1.links.push({
                    destination: p2,
                    opacity: opacityLine,
                });
            }
        }
    }
    setColor(p1) {
        const container = this.container;
        const linksOptions = p1.options.links;
        let linkColor = linksOptions.id === undefined
            ? container.particles.linksColor
            : container.particles.linksColors.get(linksOptions.id);
        if (!linkColor) {
            const optColor = linksOptions.color;
            linkColor = Utils_1.getLinkRandomColor(optColor, linksOptions.blink, linksOptions.consent);
            if (linksOptions.id === undefined) {
                container.particles.linksColor = linkColor;
            }
            else {
                container.particles.linksColors.set(linksOptions.id, linkColor);
            }
        }
    }
    getDistance(pos1, pos2, optDistance, canvasSize, warp) {
        let distance = Utils_1.NumberUtils.getDistance(pos1, pos2);
        if (distance <= optDistance || !warp) {
            return distance;
        }
        const pos2NE = {
            x: pos2.x - canvasSize.width,
            y: pos2.y,
        };
        distance = Utils_1.NumberUtils.getDistance(pos1, pos2NE);
        if (distance <= optDistance) {
            return distance;
        }
        const pos2SE = {
            x: pos2.x - canvasSize.width,
            y: pos2.y - canvasSize.height,
        };
        distance = Utils_1.NumberUtils.getDistance(pos1, pos2SE);
        if (distance <= optDistance) {
            return distance;
        }
        const pos2SW = {
            x: pos2.x,
            y: pos2.y - canvasSize.height,
        };
        distance = Utils_1.NumberUtils.getDistance(pos1, pos2SW);
        return distance;
    }
}
exports.Linker = Linker;
