"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Repulser = void 0;
const Utils_1 = require("../../Utils");
const ParticlesBase_1 = require("./ParticlesBase");
class Repulser extends ParticlesBase_1.ParticlesBase {
    constructor(container) {
        super(container, "repulser");
    }
    isEnabled(particle) {
        return particle.options.repulse.enabled;
    }
    reset() {
    }
    interact(p1) {
        const container = this.container;
        const repulseOpt1 = p1.options.repulse;
        const pos1 = p1.getPosition();
        const query = container.particles.quadTree.queryCircle(pos1, repulseOpt1.distance);
        for (const p2 of query) {
            if (p1 === p2 || p2.destroyed) {
                continue;
            }
            const pos2 = p2.getPosition();
            const { dx, dy, distance } = Utils_1.NumberUtils.getDistances(pos2, pos1);
            const velocity = repulseOpt1.speed * repulseOpt1.factor;
            if (distance > 0) {
                const repulseFactor = Utils_1.NumberUtils.clamp((1 - Math.pow(distance / repulseOpt1.distance, 2)) * velocity, 0, velocity);
                const normVec = {
                    x: (dx / distance) * repulseFactor,
                    y: (dy / distance) * repulseFactor,
                };
                p2.mover.moveXY(normVec.x, normVec.y);
            }
            else {
                p2.mover.moveXY(velocity, velocity);
            }
        }
    }
}
exports.Repulser = Repulser;
