"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LifeUpdater = void 0;
const Utils_1 = require("../Utils");
class LifeUpdater {
    constructor(container) {
        this.container = container;
    }
    isEnabled(particle) {
        return !particle.destroyed;
    }
    update(particle, delta) {
        if (!this.isEnabled(particle)) {
            return;
        }
        let justSpawned = false;
        if (particle.spawning) {
            particle.lifeDelayTime += delta.value;
            if (particle.lifeDelayTime >= particle.lifeDelay) {
                justSpawned = true;
                particle.spawning = false;
                particle.lifeDelayTime = 0;
                particle.lifeTime = 0;
            }
            else {
                return;
            }
        }
        if (particle.lifeDuration === -1) {
            return;
        }
        if (justSpawned) {
            particle.lifeTime = 0;
        }
        else {
            particle.lifeTime += delta.value;
        }
        if (particle.lifeTime < particle.lifeDuration) {
            return;
        }
        particle.lifeTime = 0;
        if (particle.livesRemaining > 0) {
            particle.livesRemaining--;
        }
        if (particle.livesRemaining === 0) {
            particle.destroy();
            return;
        }
        const canvasSize = this.container.canvas.size;
        particle.position.x = Utils_1.NumberUtils.randomInRange(0, canvasSize.width);
        particle.position.y = Utils_1.NumberUtils.randomInRange(0, canvasSize.height);
        particle.spawning = true;
        particle.lifeDelayTime = 0;
        particle.lifeTime = 0;
        particle.reset();
        const lifeOptions = particle.options.life;
        particle.lifeDelay = Utils_1.NumberUtils.getValue(lifeOptions.delay) * 1000;
        particle.lifeDuration = Utils_1.NumberUtils.getValue(lifeOptions.duration) * 1000;
    }
}
exports.LifeUpdater = LifeUpdater;
