"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpacityUpdater = void 0;
const Utils_1 = require("../Utils");
class OpacityUpdater {
    constructor(container) {
        this.container = container;
    }
    isEnabled(particle) {
        const sizeOpt = particle.options.size;
        const sizeAnim = sizeOpt.animation;
        return (!particle.destroyed &&
            !particle.spawning &&
            sizeAnim.enable &&
            (sizeAnim.count <= 0 || particle.loops.size < sizeAnim.count));
    }
    update(particle, delta) {
        var _a, _b;
        const container = this.container;
        const sizeOpt = particle.options.size;
        const sizeAnim = sizeOpt.animation;
        const sizeVelocity = ((_a = particle.size.velocity) !== null && _a !== void 0 ? _a : 0) * delta.factor;
        const maxValue = (_b = particle.sizeValue) !== null && _b !== void 0 ? _b : container.retina.sizeValue;
        const minValue = sizeAnim.minimumValue * container.retina.pixelRatio;
        if (!this.isEnabled(particle)) {
            return;
        }
        switch (particle.size.status) {
            case 0:
                if (particle.size.value >= maxValue) {
                    particle.size.status = 1;
                    particle.loops.size++;
                }
                else {
                    particle.size.value += sizeVelocity;
                }
                break;
            case 1:
                if (particle.size.value <= minValue) {
                    particle.size.status = 0;
                    particle.loops.size++;
                }
                else {
                    particle.size.value -= sizeVelocity;
                }
        }
        Utils_1.Utils.checkDestroy(particle, sizeAnim.destroy, particle.size.value, minValue, maxValue);
        if (!particle.destroyed) {
            particle.size.value = Utils_1.NumberUtils.clamp(particle.size.value, minValue, maxValue);
        }
    }
}
exports.OpacityUpdater = OpacityUpdater;
