"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SizeUpdater = void 0;
const Utils_1 = require("../Utils");
class SizeUpdater {
    constructor(container) {
        this.container = container;
    }
    isEnabled(particle) {
        const opacityAnim = particle.options.opacity.anim;
        return (!particle.destroyed &&
            !particle.spawning &&
            opacityAnim.enable &&
            (opacityAnim.count <= 0 || particle.loops.opacity < opacityAnim.count));
    }
    update(particle, delta) {
        var _a, _b;
        const opacityAnim = particle.options.opacity.anim;
        const minValue = opacityAnim.minimumValue;
        const maxValue = particle.options.opacity.value;
        if (!this.isEnabled(particle)) {
            return;
        }
        switch (particle.opacity.status) {
            case 0:
                if (particle.opacity.value >= maxValue) {
                    particle.opacity.status = 1;
                    particle.loops.opacity++;
                }
                else {
                    particle.opacity.value += ((_a = particle.opacity.velocity) !== null && _a !== void 0 ? _a : 0) * delta.factor;
                }
                break;
            case 1:
                if (particle.opacity.value <= minValue) {
                    particle.opacity.status = 0;
                    particle.loops.opacity++;
                }
                else {
                    particle.opacity.value -= ((_b = particle.opacity.velocity) !== null && _b !== void 0 ? _b : 0) * delta.factor;
                }
                break;
        }
        Utils_1.Utils.checkDestroy(particle, opacityAnim.destroy, particle.opacity.value, minValue, maxValue);
        if (!particle.destroyed) {
            particle.opacity.value = Utils_1.NumberUtils.clamp(particle.opacity.value, minValue, maxValue);
        }
    }
}
exports.SizeUpdater = SizeUpdater;
