import type { Container } from "./Container";
import type { IVelocity } from "./Interfaces/IVelocity";
import type { IParticleValueAnimation } from "./Interfaces/IParticleValueAnimation";
import type { ICoordinates, ICoordinates3d } from "./Interfaces/ICoordinates";
import type { IHsl, IRgb } from "./Interfaces/Colors";
import type { IShapeValues } from "../Options/Interfaces/Particles/Shape/IShapeValues";
import type { IBubbleParticleData } from "./Interfaces/IBubbleParticleData";
import type { IParticle } from "./Interfaces/IParticle";
import type { IParticles } from "../Options/Interfaces/Particles/IParticles";
import { Particles } from "../Options/Classes/Particles/Particles";
import { MoveDirection, MoveDirectionAlt, RotateDirection, ShapeType } from "../Enums";
import type { RecursivePartial } from "../Types";
import type { IDelta } from "./Interfaces/IDelta";
import { Mover } from "./Particle/Mover";
import type { ILink } from "./Interfaces/ILink";
import type { IParticleLoops } from "./Interfaces/IParticleLoops";
import type { IParticleInfection } from "./Interfaces/IParticleInfection";
import type { IParticleHslAnimation } from "./Interfaces/IParticleHslAnimation";
import type { Stroke } from "../Options/Classes/Particles/Stroke";
export declare class Particle implements IParticle {
    readonly id: number;
    readonly container: Container;
    readonly group?: string | undefined;
    pathAngle: number;
    destroyed: boolean;
    lifeDelay: number;
    lifeDelayTime: number;
    lifeDuration: number;
    lifeTime: number;
    livesRemaining: number;
    misplaced: boolean;
    spawning: boolean;
    lastNoiseTime: number;
    zIndexFactor: number;
    splitCount: number;
    unbreakable: boolean;
    readonly noiseDelay: number;
    readonly mover: Mover;
    readonly sides: number;
    readonly strokeWidth: number;
    readonly options: Particles;
    links: ILink[];
    randomIndexData?: number;
    linksDistance?: number;
    attractDistance?: number;
    linksWidth?: number;
    maxDistance?: RecursivePartial<IVelocity>;
    moveSpeed?: number;
    sizeValue?: number;
    sizeAnimationSpeed?: number;
    spinRadius?: number;
    spinAngle?: number;
    spinDirection?: RotateDirection;
    spinAcceleration?: number;
    orbitRadius?: number;
    orbitRotation?: number;
    close: boolean;
    fill: boolean;
    readonly direction: MoveDirection | keyof typeof MoveDirection | MoveDirectionAlt;
    readonly infection: IParticleInfection;
    readonly loops: IParticleLoops;
    readonly stroke: Stroke;
    readonly position: ICoordinates3d;
    readonly offset: ICoordinates;
    readonly shadowColor: IRgb | undefined;
    readonly color?: IParticleHslAnimation;
    readonly opacity: IParticleValueAnimation<number>;
    readonly rotate: IParticleValueAnimation<number>;
    readonly size: IParticleValueAnimation<number>;
    readonly strokeColor?: IParticleHslAnimation;
    readonly orbitColor?: IHsl;
    readonly velocity: IVelocity;
    readonly shape: ShapeType | string;
    readonly initialPosition: ICoordinates;
    readonly initialVelocity: IVelocity;
    readonly shapeData?: IShapeValues;
    readonly bubble: IBubbleParticleData;
    readonly spinCenter?: ICoordinates;
    constructor(id: number, container: Container, position?: ICoordinates, overrideOptions?: RecursivePartial<IParticles>, group?: string | undefined);
    move(delta: IDelta): void;
    draw(delta: IDelta): void;
    getPosition(): ICoordinates3d;
    getRadius(): number;
    getFillColor(): IHsl | undefined;
    getStrokeColor(): IHsl | undefined;
    destroy(override?: boolean): void;
    reset(): void;
    private setColorAnimation;
    private calcPosition;
    private calculateVelocity;
    private split;
}
